#!/usr/bin/env python

"""
Find water bottom using any number of algorithms.  Add your own!!

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: segywaterbottom.py,v 1.6 2005/10/08 22:42:28 schwehr Exp $

"""

# These values end up in the pydoc web page
__version__ = '$Revision: 1.6 $' # See man ident
__date__ = '$Date: 2005/10/08 22:42:28 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'

import os
import sys

BOMBASTIC= 4
VERBOSE  = 3
TRACE    = 2
TERSE    = 1
ALWAYS   = 0


from segy import Segy
import segy

from optparse import OptionParser

# What version do we want to track?  This would be for cvs of this file
vcvs = "$Revision: 1.6 $"
vcvs = vcvs[vcvs.find(':')+2:]
vcvs = vcvs[:vcvs.find('$')-1]
version_cvs = vcvs
del (vcvs) # stop polution

# Track the version of segy-py.  If I use setup.py, which one gets imported?
from segy_version import VERSION

myparser = OptionParser(usage="%prog [options] segyfile1 segyfile2 ... segyfileN",
                      version="%prog "+VERSION+' - '+version_cvs)

myparser.add_option('-d','--driver',dest="drv",
                    help="specify which driver to parse data file.  [default: %default]",
                    default="segy_drv_rev1")

myparser.add_option('--database',dest='database',default=None,
                    help="Database to put the results in.  [default: %default]")
myparser.add_option('--database-key',dest='databaseKey',default='wbt',
                    help="What key to insert on as the unique key for the waterbottom pick [%default]")


myparser.add_option('-f','--file',dest="file",action="append", type="string",
                    help="specify segy file to examine.  Specify one or more of these")

myparser.add_option('-v','--verbose',action="count",dest='verbosity',default=0,
                    help='how much information to give.  Specify multiple times to increase verbosity')
myparser.add_option('--verbosity',dest='verbosity',type='int',
                    help='Specify verbosity.  Should be in the range of '
                    +str(ALWAYS)+'...'+str(BOMBASTIC)+' (None...Bombastic)')
myparser.add_option('--noisy',dest='verbosity', action='store_const', const=2*BOMBASTIC,
                    help='Go for the max verbosity')

myparser.add_option('-t','--threshold',dest='threshold',action='append',type='float',
                    help='Trigger when a threshold level is reached.  Okay to specify multiple thresholds')

myparser.add_option('-s','--skip',dest='skip',type='int',
                    help='How many traces to skip between those that we check', default=0)

myparser.add_option('-T','--time',dest='time',action='store_true', default=False,
                    help='Return matches as time in seconds rather than trace data offset')

# FIX: make these three be a choice -T, -D, or trace space
myparser.add_option('-D','--depth',dest='depth',action='store_true', default=False,
                    help='Return matches as depth in meters assuming 1500 m/s')

myparser.add_option('-O','--offset',dest='offset',type='float',default=0,
                    help='Fish depth offset.  Positive number of the meters below sea level.'+
                    ' [default: %default]')

myparser.add_option('-o','--outfile',dest='outfile',type='string',default='wbt.dat',
                    help='File to write out results to.'+
                    ' [default: %default]')

myparser.add_option('-n','--negative',dest='negative',action='store_true', default=False,
                    help='Makes depths negaive so that < 0 is below sea level')



myparser.add_option('-p','--position',dest='position', action='store_true', default=False,
                    help='Add the long lat to the output [ THIS IS A HACK ]')

myparser.add_option('-k','--key',dest='key',type='string', default=None,
                    help='Reference key to put in the 2nd column')


myparser.add_option('--trace-key',dest='traceKey',type='string',default=None,
                    help='Which key to use for limiting trace range.  Shotpoint, EnsembleNo, '
                    +'LineSeqNo, etc. [default: %default] ')
myparser.add_option('--trace-min',dest='traceRangeMin',type='int',default=None,
                    help='The first number to include [ default: %default ]')
myparser.add_option('--trace-max',dest='traceRangeMax',type='int',default=None,
                    help='Last number to include  [ default: %default ]')


# FIX: what is the best output format?
# FIX: make each algorithm be a subclass of a parent class or something.
# What kind of API?  Want access to neighbors.  Generic or depend on segy class?

def offset2meters(segy, trace, offset):
    time = offset2seconds(segy, trace, offset)
    depth = time * 750.  # time is TWTT, so use 1500/2
    return depth

def offset2seconds(segy, trace, offset):
    """Convert the data array offset in the trace data block to a two way travel time

    segy - Segy data file object
    trace - trace number starting from 1
    offset - what bin the sample is in
    """
    #import pdb
    #pdb.set_trace()
    
    #print 
    traceHeader = segy.getTraceHdr(trace)

    #from IPython.Shell import IPShellEmbed
    #ipshell = IPShellEmbed(argv=[])
    #ipshell() 

    interval = traceHeader.getHeader('SampleInterval')/1000000.
    delay    = traceHeader.getHeader('Delay')/1000.
    try:
        time = delay + interval * offset
    except TypeError:
        print 'ERROR in offset2seconds:', trace, offset, type(delay),type(interval), type(delay)
        print '  ... ', delay, interval, offset
        time = delay
    return time


import sqlite
from segysqldump import hasField  # FIX: Wrong place for sqlite hasField

if __name__ == '__main__':
    (options,args) = myparser.parse_args()
    v = options.verbosity
    if v >= VERBOSE:   print '# Option values: ', options
    if v >= BOMBASTIC: print '# Arg    values: ', args

    if len(options.file)<1:
        sys.exit("ERROR: must specify one or more files with --file")

    if options.time and options.depth:
        sys.exit("ERROR: can't currently do both depth and time")

    cx = None
    cu = None
    if options.database!=None:
        cx = sqlite.connect(options.database)
        cu = cx.cursor()

    out = open (options.outfile,'w')

    # FIX: make each bottom feeder its own function!!!!!
    for file in options.file:
        print 'FILE: ',file
        out.write('# FILE: '+file+'\n')
        out.write('# traceNumber ')
        if options.position: out.write(' long lat ')
        if None != options.key: out.write (options.key)
        if options.time: out.write(' time(s) ')
        if options.depth: out.write(' depth(m) ')
        out.write('\n\n')

        fileKey = None
        if None!=cu:
            cu.execute('SELECT fileKey FROM segyFile WHERE filename="'+file+'" limit 1;')
            fileKey = cu.fetchone()['fileKey']

        s = Segy(file,drivername=options.drv,verbosity=options.verbosity)
        for traceNumber in range(1,s.getNumberOfTraces()+1):
            if (traceNumber%500==0):
                if options.verbosity>=TRACE: print "committing to traceNumber ",traceNumber
                if options.database != None:
                    cx.commit()
            if options.verbosity>=TRACE:
                if (traceNumber%10==0): print '  traceNumber ',traceNumber

            if (0<options.skip) and (traceNumber%options.skip != 0): continue

            # Only process a small range
            if None!=options.traceKey:
                val = s.getTraceHdr(traceNumber).getHeader(options.traceKey)

                if None!=options.traceRangeMin:
                    if val < options.traceRangeMin:
                        if options.verbosity>BOMBASTIC+1: print 'SKIPPING: below range'
                        continue
                    if val > options.traceRangeMax:
                        if options.verbosity>BOMBASTIC+1: print 'SKIPPING: above range'
                        continue

            data = s.getTraceData(traceNumber)

            thresholdPositions=[]
            for thresh in options.threshold:
                result = None
                for position in range(len(data)):
                    #print position, data[position], thresh
                    if data[position]>thresh:
                        result=position
                        break
                thresholdPositions.append(result)
            out.write(str(traceNumber)+' ')
            if options.position:
                # FIX: use a real coordinate transform!!!!! THIS IS A FUGLY HACK
                x = s.getTraceHdr(traceNumber).getHeader('X')/3600.
                y = s.getTraceHdr(traceNumber).getHeader('Y')/3600.
                #print x, y,
                out.write (str(x)+' '+str(y)+' ')
            if options.key != None:
                keyval = s.getTraceHdr(traceNumber).getHeader(options.key)
                out.write(str(keyval)+' ')

            for p in thresholdPositions:
                val = None
                if options.time:
                    if None != p: val = offset2seconds(s,traceNumber,p)
                elif options.depth:
                    if None != p: val = offset2meters(s,traceNumber,p)
                    val += options.offset # Fish depth
                    if options.negative: val = -val
                else:
                    val = p
                if None!=cu and val != None:
                    # Ready to update the sql database
                    updateStr = "UPDATE segyTrace SET wbt = "
                    updateStr += str(val)
                    updateStr += ' WHERE fileKey = '+str(fileKey)+' AND traceNumber = '+str(traceNumber)+';'
                    if options.verbosity>=BOMBASTIC:
                        print updateStr
                    cu.execute(updateStr)
                out.write(str(val))

            out.write('\n')

        if options.database != None:
            cx.commit()
