"""
Command utility to examine a SEGY file

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
__version__ = '$Revision: 1.1 $' # See man ident
__date__ = '$Date: 2005/10/08 22:35:18 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'



import sys

class sqlselect:
    """
    Construct an sql select query

    Sometimes it just gets ugly having all that comma and WHERE AND
    logic in there.  This code takes care of that
    """
    def __init__(self):
        self.fields = []
        self.where = []
        self.limit = None
        self.from_tables = []
        self.orderby = None
        self.desc = False # descending sort if true
        return

    def setorderby(self,field,desc=False):
        "Make the returned rows come in some order"
        if str != type(field): print "ERROR: fix throw type exception"
        self.orderby = field
        self.desc = desc
        return

    def addfield(self,fieldname):
        "Add a field name to return"
        if str != type(fieldname): print "ERROR: fix throw type exception"
        self.fields.append(fieldname)
        return

    def addwhere(self,boolTest):
        " Add expressions to chain together with ANDs"
        if str != type(boolTest):
            print "ERROR: fix throw type exception"
        self.where.append(boolTest)
        return

    def addfrom(self,tableName):
        "Which tables the query will pull from"
        if str != type(tableName):
            print "ERROR: fix throw type exception"
        self.from_tables.append(tableName)
        return

    def setlimit(self,numOfItems):
        "Set the maximum number of items to return"
        if int != type(numOfItems):
            print "ERROR: fix throw type exception"
        self.limit = numOfItems
        return

    def __str__(self):
        "Return the query as a string"
        if len(self.fields) < 1: print "ERROR: fix throw some exception?"
        s = 'SELECT '
        for i in range (len(self.fields)-1):
            s += self.fields[i]+','
        s += self.fields[-1] + ' '

        if len(self.from_tables)<1: print "ERROR: fix throw some exception"
        s += 'FROM '
        for i in range (len(self.from_tables)-1):
            s += self.from_tables[i]+','
        s += self.from_tables[-1]

        if (len(self.where)>0): s += ' WHERE '
        for i in range (len(self.where)-1):
            s += self.where[i]+' AND '
        if (len(self.where)>0): s += self.where[-1]

        if (None != self.orderby):
            s += ' ORDER BY ' + self.orderby
            if self.desc: s += ' DESC'

        if (None != self.limit):
            s += ' LIMIT ' + str(self.limit)

        s += ';'
        return s
